from plonetheme.nuplone.tiles.tile import AppConfigTile
from plonetheme.nuplone.utils import SimpleLiteral


class StaticTile(AppConfigTile):
    def __call__(self):
        content = self.data.get("content", None)
        if not content:
            return None
        return SimpleLiteral(content)



from pkg_resources import get_distribution
from plone.memoize import forever
from Products.Five import BrowserView


class NuPloneVersionView(BrowserView):
    """Helper view to return the NuPlone version. Can be used to break caching
    for resources, like:

    oira.cms.min.js?v=${here/@@nuplone-version}
    """

    @forever.memoize
    def __call__(self):
        return get_distribution("NuPlone").version



from plone.testing.zope import Browser
from plonetheme.nuplone.testing import NUPLONE_FUNCTIONAL_TESTING

import unittest


class PasswordResetTests(unittest.TestCase):
    layer = NUPLONE_FUNCTIONAL_TESTING

    def testNoKey(self):
        browser = Browser(self.layer["app"])
        browser.open("%s/reset-password" % self.layer["portal"].absolute_url())
        browser.getForm("form").submit()

    def testInvalidKey(self):
        from zExceptions import NotFound

        browser = Browser(self.layer["app"])
        browser.handleErrors = False
        url = "%s/reset-password/bogus" % self.layer["portal"].absolute_url()
        self.assertRaises(NotFound, browser.open, url)

    def testDoubleKeys(self):
        from zExceptions import NotFound

        browser = Browser(self.layer["app"])
        browser.handleErrors = False
        url = "%s/reset-password/one/two" % self.layer["portal"].absolute_url()
        self.assertRaises(NotFound, browser.open, url)



# coding=utf-8
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneWithPackageLayer

import plonetheme.nuplone


FIXTURE = PloneWithPackageLayer(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE,),
    zcml_filename="configure.zcml",
    zcml_package=plonetheme.nuplone,
    gs_profile_id="plonetheme.nuplone:default",
    name="plonetheme.nuplone:fixture",
)

NUPLONE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name="NuPlone:Integration"
)


NUPLONE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name="NuPlone:Functional"
)


__all__ = ["NUPLONE_INTEGRATION_TESTING", "NUPLONE_FUNCTIONAL_TESTING"]



# -*- coding: UTF-8 -*-
""" StatusMessage adapter tests.
"""
from plonetheme.nuplone.testing import NUPLONE_INTEGRATION_TESTING
from Products.statusmessages.interfaces import IStatusMessage
from unittest import TestCase

import six


class TestHTMLStatusMessages(TestCase):
    layer = NUPLONE_INTEGRATION_TESTING

    def testAdapter(self):
        """Test status messages
        First some boilerplate.
        """
        request = self.layer["request"].clone()

        # Now lets make sure we can actually adapt the request.
        status = IStatusMessage(request)
        self.assertTrue(IStatusMessage.providedBy(status))

        # Make sure there's no stored message.
        self.assertEqual(len(status.show()), 0)

        # Add one message
        status.add("test", type="info")

        # Now check the results
        messages = status.show()
        self.assertEqual(len(messages), 1)
        self.assertEqual(messages[0].message, "test")
        self.assertEqual(messages[0].type, "info")

        # Make sure messages are removed
        self.assertEqual(len(status.show()), 0)

        # Since we accessed the message prior to publishing the page, we must
        # ensure that the messages have been removed from the cookies
        self.assertEqual(len(status.show()), 0)

        # Now we repeat the test, only this time we publish the page prior to
        # retrieving the messages

        # Add one message
        status.add("test", type="info")

        # Now check the results
        messages = status.show()
        self.assertEqual(len(messages), 1)
        self.assertEqual(messages[0].message, "test")
        self.assertEqual(messages[0].type, "info")

        # Make sure messages are removed
        self.assertEqual(len(status.show()), 0)

        # Add two messages
        status.add("test", type="info")
        status.add("test1", "warn")

        # And check the results again
        messages = status.show()
        self.assertEqual(len(messages), 2)
        test = messages[1]
        self.assertEqual(test.message, "test1")
        self.assertEqual(test.type, "warn")

        # Make sure messages are removed again
        self.assertEqual(len(status.show()), 0)

        # Add two identical messages
        status.add("test", type="info")
        status.add("test", type="info")

        # And check the results again
        messages = status.show()
        self.assertEqual(len(messages), 1)
        test = messages[0]
        self.assertEqual(test.message, "test")
        self.assertEqual(test.type, "info")

        # Make sure messages are removed again
        self.assertEqual(len(status.show()), 0)

        # Test incredibly long messages:
        status.add("m" * 0x400, type="t" * 0x20)

        # And check the results again
        messages = status.show()
        self.assertEqual(len(messages), 1)

        test = messages[0]
        assert test.message == "m" * 0x3FF
        assert test.type == "t" * 0x1F

        # Add one HTML messages
        status.add("test", type="success")

        # And check the results again
        messages = status.show()
        self.assertEqual(len(messages), 1)
        test = messages[0]
        self.assertEqual(test.type, "success")
        self.assertEqual(test.message, "test")
        self.assertIsInstance(test.message, six.text_type)

        # Add two HTML messages
        status.add("test", type="info")
        status.add("test1", "warn")

        # And check the results again
        messages = status.show()
        self.assertEqual(len(messages), 2)
        test = messages[1]
        self.assertEqual(test.message, "test1")
        self.assertIsInstance(test.message, six.text_type)
        self.assertEqual(test.type, "warn")

        # add two identical messages
        status.add("test", type="info")
        status.add("test", type="info")

        # And check the results again
        messages = status.show()
        self.assertEqual(len(messages), 1)
        test = messages[0]
        self.assertEqual(test.message, "test")
        self.assertIsInstance(test.message, six.text_type)
        self.assertEqual(test.type, "info")

        # Make sure messages are removed again
        self.assertEqual(len(status.show()), 0)

        # Test incredibly long messages:
        status.add("m" * 0x400, type="t" * 0x20)

        # And check the results again
        messages = status.show()
        self.assertEqual(len(messages), 1)

        test = messages[0]
        self.assertIsInstance(test.message, six.text_type)
        assert test.message == "%s" % ("m" * 0x3FF)
        assert test.type == "t" * 0x1F

        # Add two mixed messages
        status.add("test", type="info")
        status.add("test1", "warn")

        # And check the results again
        messages = status.show()
        self.assertEqual(len(messages), 2)
        test = messages[0]
        self.assertEqual(test.message, "test")
        self.assertIsInstance(test.message, six.text_type)
        self.assertEqual(test.type, "info")

        test = messages[1]
        self.assertEqual(test.message, "test1")
        self.assertIsInstance(test.message, six.text_type)
        self.assertEqual(test.type, "warn")

        # Add a more complicated html message
        status.add('You can go <a href="http://plone.org">here</a>.', type="success")
        messages = status.show()
        self.assertEqual(len(messages), 1)
        test = messages[0]
        self.assertEqual(test.type, "success")
        self.assertEqual(
            test.message,
            'You can go <a href="http://plone.org">here</a>.',  # noqa: E501
        )
        self.assertIsInstance(test.message, six.text_type)



# coding=utf-8
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone import api
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.tiles import Tile
from plonetheme.nuplone.utils import getNavigationRoot
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import typesToList
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface

import collections


class INavtreeFactory(Interface):
    """Marker interface for catalog builder functions."""

    def __call__(context, request):
        """Return a CatalogNavTree instance."""


class CatalogNavTree(object):
    def __init__(self, context, request):
        self.build(context, request)

    def build(self, context, request):
        context = aq_inner(context)

        # If we are at a default page use the folder as context for the navtree
        container = aq_parent(context)
        isp = queryMultiAdapter((container, request), name="default_page", default=None)
        if isp is not None and isp.isDefaultPage(context):
            context = container

        contextPath = "/".join(context.getPhysicalPath())
        contextPathLen = len(contextPath)
        parentDepth = contextPath.count("/") - 1
        navrootPath = "/".join(getNavigationRoot(context).getPhysicalPath())

        query = {}
        query["path"] = dict(
            query=contextPath, navtree=True, navtree_start=navrootPath.count("/")
        )
        query["portal_type"] = typesToList(context)
        query["sort_on"] = "getObjPositionInParent"
        query["sort_order"] = "asc"

        catalog = getToolByName(context, "portal_catalog")
        results = catalog.searchResults(query)
        cache = {}
        cache[navrootPath] = {"current": False, "currentParent": True, "children": []}
        for brain in results:
            path = brain.getPath()
            pathLen = len(path)
            parentPath = path.rsplit("/", 1)[0]
            ancestor = current = currentParent = False
            if path == contextPath:
                current = True
            elif contextPathLen > pathLen:
                ancestor = contextPath.startswith(path + "/")
                currentParent = ancestor and path.count("/") == parentDepth

            if brain.exclude_from_nav and not currentParent:
                continue

            oldNode = cache.get(path, None)
            node = {
                "brain": brain,
                "path": path,
                "current": current,
                "currentParent": currentParent,
                "ancestor": ancestor,
            }

            oldNode = cache.get(path, None)
            if oldNode is not None:
                oldNode.update(node)
                node = oldNode
            else:
                node["children"] = []
                cache[path] = node

            parentNode = cache.get(parentPath, None)
            if parentNode is None:
                parentNode = cache[parentPath] = dict(children=[node])
            else:
                parentNode["children"].append(node)
            node["parent"] = parentNode

        self.tree = cache
        self.root = cache[navrootPath]

    def __iter__(self):
        """Breadth-first iterator for navtree nodes which allows
        modifications of the tree during iteration. Modifications
        are given by passing a command to the next() method of the
        generator. For example:

        >>> tree=CatalogNavTree(context, request)
        >>> g=tree.iter()
        >>> value=next(g)
        >>> try:
        ...     while True:
        ...        if value.portal_type=="Collection":
        ...            value=g.send("prune")
        ...        else:
        ...            value=next(g)
        ... except StopIteration:
        ...     pass

        The supported commands are:

        * purge: remove the node and all its children from the tree
        * prune: remove all children of this node from the tree

        In addition you may also modify the datastructures directly
        during iteration.
        """
        queue = collections.deque([self.root])
        while queue:
            node = queue.popleft()
            action = yield node
            if action == "purge":
                node["parent"]["children"].remove(node)
                continue
            elif action == "prune":
                node["children"] = []
                continue

            for child in node.get("children", []):
                queue.append(child)

    iter = __iter__


@implementer(INavtreeFactory)
@adapter(Interface, Interface)
class TreeFactory(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        return CatalogNavTree(self.context, self.request)


class NavigationTile(Tile):
    def update(self):
        portal_types = getToolByName(self.context, "portal_types")
        type_titles = dict(
            [(fti.getId(), fti.Title()) for fti in portal_types.listTypeInfo()]
        )

        use_view_types = api.portal.get_registry_record(
            "plone.types_use_view_action_in_listings", default=[]
        )
        normalize = getUtility(IIDNormalizer).normalize
        treefactory = getMultiAdapter((self.context, self.request), INavtreeFactory)
        tree = treefactory()

        for node in tree.iter():
            brain = node.get("brain", None)
            if brain is None:
                continue
            node["title"] = brain.Title
            node["description"] = brain.Description or None
            node["portal_type"] = normalize(brain.portal_type)
            node["portal_type_title"] = type_titles.get(
                brain.portal_type, brain.portal_type
            )
            node["url"] = (
                "%s/view" % brain.getURL()
                if brain.portal_type in use_view_types
                else brain.getURL()
            )
            node["review_state"] = normalize(brain.review_state)
            node["folderish"] = brain.is_folderish
            node["class"] = (
                " ".join(
                    filter(
                        None,
                        [
                            "active"
                            if node["current"] or node["currentParent"]
                            else None,
                            "current" if node["current"] else None,
                        ],
                    )
                )
                or None
            )

        if "brain" in tree.root:
            self.tree = [tree.root]
        else:
            self.tree = tree.root["children"]

    def __call__(self):
        self.update()
        return self.index()



from plone.formwidget.namedfile.widget import NamedFileWidget
from plone.formwidget.namedfile.widget import NamedImageWidget
from plone.namedfile.interfaces import INamedFileField
from plone.namedfile.interfaces import INamedImageField
from plonetheme.nuplone.z3cform.interfaces import INuPloneFormLayer
from plonetheme.nuplone.z3cform.utils import getVocabulary
from Products.Five import BrowserView
from z3c.form.browser.file import FileWidget
from z3c.form.browser.radio import RadioWidget
from z3c.form.browser.select import SelectWidget
from z3c.form.interfaces import IDataManager
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import NOVALUE
from z3c.form.widget import FieldWidget
from ZODB.POSException import POSKeyError
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.schema.interfaces import IChoice
from ZPublisher.HTTPRequest import FileUpload


class SingleRadioWidget(RadioWidget):
    """Variant of the z3c.form radio widget which does not pretend a radio
    field can ever return more than one value.

    self.value is still a list since this widget uses the
    SequenceDataConverter. Apparently z3c.form does this since it has no
    concept of a single-valued field which takes its value from a
    vocabulary.
    """

    def update(self):
        super(SingleRadioWidget, self).update()
        for item in self.items:
            item["name"] = item["name"].split(":list", 1)[0]


@adapter(IChoice, INuPloneFormLayer)
@implementer(IFieldWidget)
def ChoiceWidgetFactory(field, request):
    vocabulary = getVocabulary(field)
    if vocabulary is None or len(vocabulary) > 4:
        widget = SelectWidget
    else:
        widget = SingleRadioWidget
    return FieldWidget(field, widget(request))


class NewMultiWidgetEntry(BrowserView):
    def __call__(self):
        widget = self.context.getWidget(0)
        return widget.render()


class NicerNamedImageWidget(NamedImageWidget):
    @property
    def allow_nochange(self):
        # Prevent errors caused by missing blob file
        if self.value != self.field.missing_value:
            try:
                blob = getattr(self.value, "_blob", None)
                if blob:
                    blob.__repr__()
            except POSKeyError:
                return False
        return super(NicerNamedImageWidget, self).allow_nochange

    def extract(self, default=NOVALUE):
        action = self.request.get("%s.action" % self.name, None)
        if action == "remove":
            return None

        value = FileWidget.extract(self, default)

        if value is NOVALUE or (isinstance(value, FileUpload) and not value.filename):
            if self.ignoreContext:
                return default

            dm = getMultiAdapter(
                (
                    self.context,
                    self.field,
                ),
                IDataManager,
            )
            return dm.get()

        # Note that we allow the user to upload an empty file.
        return value


@adapter(INamedImageField, INuPloneFormLayer)
@implementer(IFieldWidget)
def NamedImageWidgetFactory(field, request):
    return FieldWidget(field, NicerNamedImageWidget(request))


class NicerNamedFileWidget(NamedFileWidget):
    @property
    def allow_nochange(self):
        # Prevent errors caused by missing blob file
        if self.value != self.field.missing_value:
            try:
                blob = getattr(self.value, "_blob", None)
                if blob:
                    blob.__repr__()
            except POSKeyError:
                return False
        return super(NicerNamedFileWidget, self).allow_nochange

    def extract(self, default=NOVALUE):
        action = self.request.get("%s.action" % self.name, None)
        if action == "remove":
            return None

        value = FileWidget.extract(self, default)

        if value is NOVALUE or (isinstance(value, FileUpload) and not value.filename):
            if self.ignoreContext:
                return default

            dm = getMultiAdapter(
                (
                    self.context,
                    self.field,
                ),
                IDataManager,
            )
            return dm.get()

        # Note that we allow the user to upload an empty file.
        return value


@adapter(INamedFileField, INuPloneFormLayer)
@implementer(IFieldWidget)
def NamedFileWidgetFactory(field, request):
    return FieldWidget(field, NicerNamedFileWidget(request))



from plone.autoform.form import AutoExtensibleForm
from plone.behavior.interfaces import IBehavior
from plone.dexterity.interfaces import IDexterityContent
from plone.dexterity.interfaces import IDexterityFTI
from plone.supermodel import model
from plone.supermodel.directives import MetadataListDirective
from plone.supermodel.interfaces import FIELDSETS_KEY
from plone.supermodel.utils import mergedTaggedValueList
from z3c.form.browser.checkbox import CheckBoxWidget
from z3c.form.interfaces import IGroup
from z3c.form.interfaces import IWidget
from zope.component import adapter
from zope.component import getUtility
from zope.component import queryUtility
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserView

import collections
import plone.z3cform.fieldsets.interfaces


Dependency = collections.namedtuple("Dependency", "name field op value action")
DEPENDENCY_KEY = "plonetheme.nuplone.z3cform.dependency"
LAYOUT_KEY = "plonetheme.nuplone.z3cform.layout"


class depends(MetadataListDirective):
    """Directive used to declare a dependency on other field values."""

    key = DEPENDENCY_KEY

    def factory(self, field, name, op="on", value=None, action="show"):
        if op not in ["on", "off", "==", "!="]:
            raise ValueError("Invalid operand given")
        if action not in ["show", "enable"]:
            raise ValueError("Invalid action given")
        return [Dependency(field, name, op, value, action)]


@adapter(Interface, Interface, AutoExtensibleForm)
@implementer(plone.z3cform.fieldsets.interfaces.IFormExtender)
class FormDependencyExtender(object):

    order = 0

    def __init__(self, context, request, form):
        self.context = context
        self.request = request
        self.form = form

    def update(self):
        schemas = [self.form.schema]

        if IDexterityContent.providedBy(self.context):
            fti = getUtility(IDexterityFTI, name=self.context.portal_type)
            for name in fti.behaviors:
                behavior = queryUtility(IBehavior, name=name)
                if behavior and behavior.interface.extends(model.Schema):
                    schemas.append(behavior.interface)

        directives = []
        for schema in schemas:
            directives += mergedTaggedValueList(schema, DEPENDENCY_KEY)

        dependencies = {}
        for directive in directives:
            dependencies.setdefault(directive.name, []).append(directive)

        todo = collections.deque([self.form])
        while todo:
            group = todo.pop()
            if hasattr(group, "groups"):
                todo.extendleft(group.groups)
            for (name, field) in group.fields.items():
                depends = dependencies.get(name, None)
                if depends is None:
                    continue
                field.field._dependencies = depends


@adapter(IWidget, Interface)
@implementer(IBrowserView)
class WidgetDependencyView(object):
    def __init__(self, widget, request):
        self.widget = widget
        self.request = request

    def __call__(self):
        dependencies = getattr(self.widget.field, "_dependencies", None)
        if not dependencies:
            return None

        classes = []
        widgets = self.widget.__parent__
        for dependency in dependencies:
            widget = widgets[dependency.field]
            name = widget.name
            if isinstance(widget, CheckBoxWidget):
                name = "%s:list" % name
            if dependency.op in ["on", "off"]:
                classes.append("dependsOn-%s-%s" % (name, dependency.op))
            elif dependency.op == "==":
                classes.append("dependsOn-%s-equals-%s" % (name, dependency.value))
            elif dependency.op == "!=":
                classes.append("dependsOn-%s-notEquals-%s" % (name, dependency.value))

            classes.append("dependsAction-%s" % dependency.action)

        return " ".join(classes) if classes else None


@adapter(Interface, Interface, AutoExtensibleForm)
@implementer(plone.z3cform.fieldsets.interfaces.IFormExtender)
class FormLayoutExtender(object):

    order = 10

    def __init__(self, context, request, form):
        self.context = context
        self.request = request
        self.form = form

    def update(self):
        fieldsets = mergedTaggedValueList(self.form.schema, FIELDSETS_KEY)
        if not isinstance(self.form.groups, list):
            self.form.groups = list(self.form.groups)
        groups = dict(
            [
                (group.__name__, (index, group))
                for (index, group) in enumerate(self.form.groups)
            ]
        )
        for fieldset in fieldsets:
            layout = getattr(fieldset, "layout", None)
            if layout is None:
                continue
            if fieldset.__name__ not in groups:
                continue
            (index, group) = groups[fieldset.__name__]

            if not IGroup.providedBy(group):
                group = self.form.groups[index] = group(
                    self.context, self.request, self.form
                )
            group.layout = layout



# coding=utf-8
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from plonetheme.nuplone.testing import NUPLONE_FUNCTIONAL_TESTING
from plonetheme.nuplone.testing import NUPLONE_INTEGRATION_TESTING
from zope.interface import alsoProvides

import unittest
import zope.component


class SiteMenuIntegrationTests(unittest.TestCase):
    layer = NUPLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.portal.REQUEST
        alsoProvides(self.request, NuPloneSkin)

    def test_settings_url(self):
        """Check that the sitemenu view provides a settings_url attribute"""
        view = zope.component.getMultiAdapter(
            (self.portal, self.request),
            name="sitemenu",
        )
        self.assertEqual(view.settings_url, "http://nohost/plone/@@settings")


class SiteMenuTests(unittest.TestCase):
    layer = NUPLONE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.portal.REQUEST
        alsoProvides(self.request, NuPloneSkin)
        login(self.layer["app"], SITE_OWNER_NAME)

        # Create two folders
        self.portal.invokeFactory("Folder", "source_folder")
        self.source_folder = self.portal["source_folder"]

        self.portal.invokeFactory("Folder", "dest_folder")
        self.dest_folder = self.portal["dest_folder"]

        # Create an object to be copied
        self.source_folder.invokeFactory("Document", "doc")

    def testActions(self):
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ("Contributor",))
        # Copy the object
        cp = self.source_folder.manage_copyObjects(ids=["doc"])
        self.request["__cp"] = cp

        # We cannot paste the object in the site root
        v = zope.component.getMultiAdapter((self.portal, self.request), name="sitemenu")
        self.assertIsNone(v.organise())
        # XXX it seems we have no actions here
        return
        # We can however paste the object in each of the folders
        for folder in [self.source_folder, self.dest_folder]:
            v = zope.component.getMultiAdapter((folder, self.request), name="sitemenu")
            menu = v.organise()
            self.assertIsNotNone(menu)
            children_titles = [i["title"] for i in menu["children"]]
            self.assertTrue("menu_paste" in children_titles)

        # Now when we give the user the Viewer role, they cannot paste anymore
        # so we should not see "Paste" as an available action.
        setRoles(self.portal, TEST_USER_ID, ("Viewer",))
        for folder in [self.source_folder, self.dest_folder]:
            v = zope.component.getMultiAdapter((folder, self.request), name="sitemenu")
            menu = v.organise()
            self.assertIsNotNone(menu)
            children_titles = [i["title"] for i in menu["children"]]
            self.assertFalse("menu_paste" in children_titles)



# coding=utf-8
from plone import api
from plonetheme.nuplone.auth import LoginChallenger

import logging


log = logging.getLogger(__name__)


def post_install(context):
    addNuploneChallengerPlugin()
    api.portal.set_registry_record(
        "plone.app.theming.interfaces.IThemeSettings.enabled",
        False,
    )


def addNuploneChallengerPlugin():
    site = api.portal.get()
    pas = site.acl_users
    if not pas.objectIds([LoginChallenger.meta_type]):
        plugin = LoginChallenger("nuplone-challenger", LoginChallenger.meta_type)
        pas._setObject(plugin.getId(), plugin)
        plugin = getattr(pas, plugin.getId())

        infos = [
            info
            for info in pas.plugins.listPluginTypeInfo()
            if plugin.testImplements(info["interface"])
        ]
        plugin.manage_activateInterfaces([info["id"] for info in infos])
        for info in infos:
            for i in range(len(pas.plugins.listPluginIds(info["interface"]))):
                pas.plugins.movePluginsUp(info["interface"], [plugin.getId()])



# coding=utf-8
from setuptools import find_packages
from setuptools import setup

import os


version = "2.1.2"

setup(
    name="NuPlone",
    version=version,
    description="A new user interface for Plone",
    long_description=open("README.rst").read()
    + "\n"
    + open(os.path.join("docs", "changes.rst")).read(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 5.2",
        "Framework :: Zope",
        "Framework :: Zope :: 4",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Programming Language :: JavaScript",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    keywords="",
    author="Cornelis Kolbach and Wichert Akkerman",
    author_email="wichert@wiggy.net",
    url="https://github.com/euphorie/NuPlone",
    license="GPL",
    packages=find_packages(exclude=["ez_setup"]),
    namespace_packages=["plonetheme"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "setuptools",
        "Plone >=5.1",
        "Zope2 >=2.12.5",
        "plone.api",
        "plone.tiles",
        "plone.z3cform",
        "plone.app.z3cform >=0.4.10dev",
        "z3c.form [extra] >=2.3.4dev",
        "plone.i18n",
        "plone.autoform",
        "plone.supermodel",
        "plone.formwidget.namedfile >1.0b4",
        "Products.statusmessages",
        "zope.i18n",
        "plone.api",
        "plone.dexterity",
    ],
    extras_require={
        "tests": [
            "plone.testing",
            "plone.app.robotframework",
            "plone.app.testing",
        ],
    },
)



# coding=utf-8
from Acquisition import aq_inner
from plone import api
from plone.autoform.form import AutoExtensibleForm
from plone.supermodel import model
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.utils import createEmailTo
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.PasswordResetTool import ExpiredRequestError
from Products.CMFPlone.PasswordResetTool import InvalidRequestError
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.MailHost.MailHost import MailHostError
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import form
from z3c.form.button import buttonAndHandler
from zope import schema
from zope.i18n import translate

import logging
import socket


log = logging.getLogger(__name__)


class IRequestPasswordReset(model.Schema):
    login = schema.TextLine(title=_("label_login", default="Login"), required=True)


class IPasswordReset(model.Schema):
    login = schema.TextLine(title=_("label_login", default="Login"), required=True)

    password = schema.Password(
        title=_("label_password", default="Password"), required=True
    )


class RequestPasswordForm(AutoExtensibleForm, form.Form):

    email_template = ViewPageTemplateFile("templates/pwreset-email.pt")

    ignoreContext = True
    schema = IRequestPasswordReset
    label = _("header_password_reset_request", default="Reset password")
    default_fieldset_label = None
    description = _(
        "intro_password_reset_request",
        default="For security reasons, we store your password encrypted, "
        "and cannot mail it to you. If you would like to reset "
        "your password, fill out the form below and we will send "
        "you an email at the address you gave when you registered "
        "to start the process of resetting your password.",
    )

    @property
    def email_from_name(self):
        return api.portal.get_registry_record("plone.email_from_name")

    @property
    def email_from_address(self):
        return api.portal.get_registry_record("plone.email_from_address")

    @buttonAndHandler(_("button_send", default="Send"), name="send")
    def handleSend(self, action):
        flash = IStatusMessage(self.request).addStatusMessage
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        pas = getToolByName(self.context, "acl_users")
        mt = getToolByName(self.context, "portal_membership")
        user = pas.getUser(data["login"])
        if user is None:
            log.info("Password reset request for unknown user %s" % data["login"])
            flash(_("error_pwreset_unknown_user", default="Unknown username."), "error")
            return

        member = mt.getMemberById(user.getId())
        email_address = member.getProperty("email")
        if not email_address:
            flash(
                _(
                    "error_pwreset_no_email",
                    default="No known email address for this user.",
                ),
                "error",
            )
            return

        ppr = getToolByName(self.context, "portal_password_reset")
        reset = ppr.requestReset(user.getId())
        portal_url = aq_inner(self.context).absolute_url()
        reset_url = "%s/@@reset-password/%s" % (portal_url, reset["randomstring"])

        data["site"] = self.context.title
        subject = _(
            "password_reset_subject",
            default="Password reset for ${site}",
            mapping=data,
        )
        subject = translate(subject, context=self.request)
        body = self.email_template(
            reset_url=reset_url, login=data["login"], site=self.context.title
        )

        email = createEmailTo(
            self.email_from_name,
            self.email_from_address,
            None,
            email_address,
            subject,
            body,
        )

        mh = getToolByName(self.context, "MailHost")
        try:
            mh.send(email)
        except MailHostError as e:
            log.error(
                "MailHost error sending password reset form to %s: %s", email_address, e
            )
            flash(
                _(
                    "error_contactmail",
                    "An error occured while processing your contact request. Please try again later.",  # noqa: E501
                ),
                "error",
            )
            return
        except socket.error as e:
            log.error(
                "Socket error sending password reset form to: %s", email_address, e[1]
            )
            flash(
                _(
                    "error_contactmail",
                    "An error occured while processing your contact request. Please try again later.",  # noqa: E501
                ),
                "error",
            )
            return

        flash(
            _(
                "info_pwrest_mail_sent",
                default="An email with instructions for resetting your password has been sent.",  # noqa: E501
            ),
            "success",
        )
        self.request.response.redirect(portal_url)


class PasswordReset(AutoExtensibleForm, form.Form):

    randomstring = None

    ignoreContext = True
    schema = IPasswordReset
    label = _("header_password_reset", default="Reset password")
    description = _(
        "intro_password_reset",
        default="Please fill out the form below to set your password.",
    )
    default_fieldset_label = None

    def publishTraverse(self, request, name):
        if self.randomstring is not None:
            raise KeyError(name)

        ppr = getToolByName(self.context, "portal_password_reset")
        try:
            ppr.verifyKey(name)
        except (InvalidRequestError, ExpiredRequestError):
            raise KeyError(name)

        self.randomstring = name
        return self

    @buttonAndHandler(_("button_change", default="Change"), name="change")
    def handleChange(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        flash = IStatusMessage(self.request).addStatusMessage
        pas = getToolByName(self.context, "acl_users")
        ppr = getToolByName(self.context, "portal_password_reset")
        user = pas.getUser(data["login"])
        if user is None:
            flash(
                _(
                    "user_name_wrong",
                    "The login name you have provided does not match the username from the password-reset email. Please check your spelling.",  # noqa: E501
                ),
                "error",
            )
            return
        try:
            ppr.resetPassword(user.getId(), self.randomstring, data["password"])
        except InvalidRequestError:
            flash(
                _(
                    "user_name_wrong",
                    "The login name you have provided does not match the username from the password-reset email. Please check your spelling.",  # noqa: E501
                ),
                "error",
            )
            return

        flash(_("password_reset", "Your password has been reset."), "success")
        portal_url = aq_inner(self.context).absolute_url()
        self.request.response.redirect("%s/@@login" % portal_url)



from plone.tiles import Tile
from plonetheme.nuplone.utils import viewType
from Products.CMFCore.utils import getToolByName


class LanguageTile(Tile):
    def update(self):
        lt = getToolByName(self.context, "portal_languages")
        languages = [
            _ for _ in lt.getAvailableLanguageInformation().values() if _["selected"]
        ]
        languages.sort(key=lambda l: l.get("native", l["name"]))
        self.languages = languages
        self.current_language = lt.getPreferredLanguage()

    def __call__(self):
        view_type = viewType(self.context, self.request)
        if view_type in ["add", "edit"]:
            return None

        self.update()
        return self.index()



# coding=utf-8
from AccessControl import getSecurityManager
from Acquisition import aq_chain
from Acquisition import aq_inner
from email.header import Header
from email.mime.text import MIMEText
from plone.app.layout.navigation.interfaces import INavigationRoot
from plonetheme.nuplone import MessageFactory as _
from Products.CMFCore.ActionInformation import ActionInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from z3c.form.interfaces import IAddForm
from z3c.form.interfaces import IEditForm

import collections
import email.utils as emailutils
import logging
import six


log = logging.getLogger(__name__)


def getPortal(context):
    for obj in aq_chain(aq_inner(context)):
        if IPloneSiteRoot.providedBy(obj):
            return obj


def getNavigationRoot(context):
    for obj in aq_chain(aq_inner(context)):
        if INavigationRoot.providedBy(obj):
            break
    return obj


def checkPermission(context, permission):
    return getSecurityManager().checkPermission(permission, context)


def isAnonymous(user=None):
    if user is None:
        user = getSecurityManager().getUser()
    return user is None or user.getUserName() == "Anonymous User"


def viewType(context, request):
    """Determine what type of view the user is looking at. This returns
    one of three options: ``view`` for normal object views, ``add``
    for add forms, ``edit`` for edit forms, and ``other`` for all other
    types."""

    view = request.other.get("PUBLISHED")
    if view is not None:
        if IAddForm.providedBy(view):
            return "add"
        if IEditForm.providedBy(view):
            return "edit"

    for url in ["ACTUAL_URL", "VIRTUAL_URL", "URL"]:
        current_url = request.get(url)
        if current_url is not None:
            break
    else:
        return "view"

    if current_url.endswith("/"):
        current_url = current_url[:-1]

    if current_url.endswith("@@edit"):
        return "edit"
    elif "/++add++" in current_url:
        return "add"

    try:
        if current_url == aq_inner(context).absolute_url():
            return "view"
    except AttributeError:
        # object has no absolute_url (None or Not Found)
        pass

    return "other"


class SimpleLiteral(six.text_type):
    def __html__(self):
        return six.text_type(self)


def setLanguage(request, context, lang=None):
    """Switch Plone to another language. If no language is given via the
    `lang` parameter the language is taken from a `language`
    request parameter. If a dialect was chosen but is not available the main
    language is used instead. If the main language is also not available
    nothing is done and ``False`` is returned.

    It is safe to call this method before rendering a template: it will
    immediately switch the current language. That means there is no need
    to trigger a redirect to force a new HTTP request.
    """
    if lang is None:
        lang = request.form.get("language")
    if not lang:
        return

    lang = lang.lower()
    lt = getToolByName(context, "portal_languages")
    res = lt.setLanguageCookie(lang=lang, request=request)
    if res is None and "-" in lang:
        lang = lang.split("-")[0]
        res = lt.setLanguageCookie(lang=lang, request=request)
        if res is None:
            log.warning("Failed to switch language to %s", lang)
            return False

    # In addition to setting the cookie also update the PTS language.
    # This effectively switches Plone over to the new language without
    # requiring a new HTTP request.
    request["LANGUAGE"] = lang
    binding = request.get("LANGUAGE_TOOL", None)
    if binding is not None:
        binding.LANGUAGE = lang

    return True


FactoryInfo = collections.namedtuple("FactoryInfo", "id title description url")


def getFactoriesInContext(context):
    """Return a list of all factories available to the current user at
    the given location."""
    context = aq_inner(context)
    ftis = context.allowedContentTypes()
    if not ftis:
        return []

    tt = getToolByName(context, "portal_types")
    ec = tt._getExprContext(context)
    actions = [ActionInfo(fti, ec) for fti in ftis]
    actions = [
        FactoryInfo(
            action.get("id"),
            action.get("title") or action.get("id"),
            action.get("description") or None,
            action["url"],
        )
        for action in actions
    ]
    return actions


def createEmailTo(
    sender_name,
    sender_email,
    recipient_name,
    recipient_email,
    subject,
    body,
    format="plain",
):
    """Create an :obj:`email.MIMEText.MIMEtext` instance for an email. This
    method will take care of adding addings a date header and message ID
    to the email, as well as quoting of non-ASCII content.
    """
    if isinstance(body, six.text_type):
        mail = MIMEText(body.encode("utf-8"), format, "utf-8")
    else:
        mail = MIMEText(body, format)

    if sender_name:
        mail["From"] = emailutils.formataddr((sender_name, sender_email))
    else:
        mail["From"] = sender_email
    if recipient_name:
        mail["To"] = emailutils.formataddr((recipient_name, recipient_email))
    else:
        mail["To"] = recipient_email
    mail["Subject"] = Header(subject.encode("utf-8"), "utf-8")
    mail["Date"] = emailutils.formatdate(localtime=True)
    mail.set_param("charset", "utf-8")

    return mail


def formatDate(request, date, length="long"):
    """Wrapper aound the zope.i18n date formatter which does not abort on
    pre-1900 dates.
    """
    if date.year < 1900:
        return _("date_to_early", default="<pre-1900-date>")
    return request.locale.dates.getFormatter("date", length).format(date)


def formatTime(request, time, length=None):
    return request.locale.dates.getFormatter("time", length).format(time)


def formatDatetime(request, timestamp, length="long"):
    """Wrapper aound the zope.i18n datetime formatter which does not abort on
    pre-1900 dates.
    """
    if timestamp.year < 1900:
        return _("date_to_early", default="<pre-1900-date>")
    if length == "long":
        return _(
            "format_datetime",
            default="${date} at ${time}",
            mapping=dict(
                date=formatDate(request, timestamp, "long"),
                time=formatTime(request, timestamp, "short"),
            ),
        )
    return request.locale.dates.getFormatter("dateTime", length).format(timestamp)



# coding=utf-8
from AccessControl import getSecurityManager
from Acquisition import aq_inner
from plone import api
from plone.memoize.view import memoize
from plonetheme.nuplone import utils
from Products.Five import BrowserView


class Tools(BrowserView):
    """Basic view to expose utilties to templates."""

    @property
    @memoize
    def user(self):
        return getSecurityManager().getUser()

    @property
    @memoize
    def anonymous(self):
        return self.user is None or self.user.getUserName() == "Anonymous User"

    @property
    @memoize
    def portal(self):
        return utils.getPortal(self.context)

    @property
    @memoize
    def portal_url(self):
        return self.portal.absolute_url()

    @property
    @memoize
    def navroot(self):
        return utils.getNavigationRoot(self.context)

    @property
    @memoize
    def navroot_url(self):
        return self.navroot.absolute_url()

    @property
    @memoize
    def context_url(self):
        return aq_inner(self.context).absolute_url()

    def render(self):
        """Little trick to make it easier to access this via from a TALES
        expression."""
        return self

    def view_type(self):
        return utils.viewType(self.context, self.request)

    def site_title(self):
        return api.portal.get_registry_record("plone.site_title")

    def formatDate(self, date, length="long"):
        return utils.formatDate(self.request, date, length)

    def formatTime(self, time, length=None):
        return utils.formatTime(self.request, time, length)

    def formatDatetime(self, timestamp, length="long"):
        return utils.formatDateTime(self.request, timestamp, length)

    def formatDecimal(self, value, length=None):
        return self.request.locale.numbers.getFormatter("decimal", length).format(value)

    def formatPercentage(self, value, length=None):
        return self.request.locale.numbers.getFormatter("percent", length).format(value)

    def countryName(self, code):
        return self.request.locale.displayNames.territories.get(code.upper())

    def languageName(self, code, default=None):
        code = code.lower()
        names = self.request.locale.displayNames.languages
        return names.get(code, default)

    def checkPermission(self, permission):
        return utils.checkPermission(self.context, permission)



from AccessControl import getSecurityManager
from plone.autoform.form import AutoExtensibleForm
from plone.supermodel import model
from plonetheme.nuplone import MessageFactory as _
from Products.CMFCore.utils import getToolByName
from Products.PluggableAuthService.interfaces.authservice import IPropertiedUser
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import form
from z3c.form.interfaces import IDataManager
from z3c.form.interfaces import NO_VALUE
from zope import schema
from zope.component import adapter
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.schema.interfaces import IField
from zope.schema.interfaces import IPassword


class ISettings(model.Schema):
    fullname = schema.TextLine(
        title=_("label_fullname", default="Full name"), required=True
    )

    email = schema.ASCIILine(
        title=_("label_email", default="Email address"), required=True
    )

    password = schema.Password(
        title=_("label_password", default="Password"), required=True
    )


@adapter(IPropertiedUser, IField)
@implementer(IDataManager)
class UserPropertyDataManager(object):
    def __init__(self, user, field):
        self.user = user
        self.field = field
        self.propname = field.__name__
        for sheet_id in user.listPropertysheets():
            sheet = user.getPropertysheet(sheet_id)
            if sheet.hasProperty(self.propname):
                self.propertysheet = sheet
                break
        else:
            self.propname = None

    def get(self):
        return self.propertysheet.getProperty(self.propname)

    def query(self, default=NO_VALUE):
        if self.propname is None:
            return default
        return self.propertysheet.getProperty(self.propname, NO_VALUE)

    def set(self, value):
        self.propertysheet.setProperty(self.user, self.propname, value)

    def canAccess(self):
        return self.propname is not None

    def canWrite(self):
        return self.propname is not None


@adapter(IPropertiedUser, IPassword)
@implementer(IDataManager)
class UserPasswordDataManager(object):
    def __init__(self, user, field):
        self.user = user
        self.field = field

    def get(self):
        raise AttributeError(self.field.__name__)

    def query(self, default=NO_VALUE):
        return default

    def set(self, value):
        if value is None:
            return IStatusMessage(getRequest()).add(
                _("Password not updated, none was specified."), type="error"
            )

        try:
            mt = getToolByName(self.user, "portal_membership")
        except AttributeError:
            return IStatusMessage(getRequest()).add(
                _("Cannot change password for Zope users, only Plone"), type="error"
            )
        else:
            mt.setPassword(value)

    def canAccess(self):
        return False

    def canWrite(self):
        return True


class Settings(AutoExtensibleForm, form.EditForm):

    schema = ISettings

    def getContent(self):
        return getSecurityManager().getUser()



# coding=utf-8
from Acquisition import aq_inner
from plone.folder.interfaces import IExplicitOrdering
from Products.Five.browser import BrowserView


class UpdateOrder(BrowserView):
    def __call__(self):
        order = self.request.form.get("order[]")
        if not order or not isinstance(order, list):
            return

        # If we get an empty element in between, it means we're in some sort
        # of interim state. Skip, so that we avoid ConflictError
        if order.count("") > 1:
            return

        orderer = IExplicitOrdering(aq_inner(self.context))
        for (pos, id) in enumerate(order):
            if id == "" or "-" not in id:
                continue
            id = id.split("-", 1)[1]
            orderer.moveObjectToPosition(id, pos)

        return
